"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isRight = exports.isLeft = exports.Right = exports.Left = exports.RIGHT_TAG = exports.LEFT_TAG = void 0;
const assertions_1 = require("../assertions/assertions");
const utils_1 = require("../utils/utils");
exports.LEFT_TAG = Symbol('TAG: Either.Left');
exports.RIGHT_TAG = Symbol('TAG: Either.Right');
const Left = (x) => ({
    TAG: exports.LEFT_TAG,
    value: x,
    fmap: () => (0, exports.Left)(x),
    apply: () => (0, exports.Left)(x),
    bind: () => (0, exports.Left)(x),
    // @ts-ignore
    toString: () => `Left (${(0, utils_1.show)(x)})`,
});
exports.Left = Left;
const Right = (x) => ({
    TAG: exports.RIGHT_TAG,
    value: x,
    fmap: (f) => (0, exports.Right)(f(x)),
    apply: (ef) => expectEither(ef).fmap((f) => f(x)),
    bind: (f) => expectEither(f(x)),
    // @ts-ignore
    toString: () => `Right (${(0, utils_1.show)(x)})`,
});
exports.Right = Right;
/**
 * Tells if the given value is a {@link Just} instance of Maybe.
 */
function isLeft(m) {
    try {
        return m.TAG === exports.LEFT_TAG
            && m.hasOwnProperty('value')
            && (0, assertions_1.isFunction)(m.fmap)
            && (0, assertions_1.isFunction)(m.apply)
            && (0, assertions_1.isFunction)(m.bind);
    }
    catch (error) {
        return false;
    }
}
exports.isLeft = isLeft;
/**
 * Tells if the given value is a {@link Nothing} instance of Maybe.
 */
function isRight(m) {
    try {
        return m.TAG === exports.RIGHT_TAG
            && (0, assertions_1.isFunction)(m.fmap)
            && (0, assertions_1.isFunction)(m.apply)
            && (0, assertions_1.isFunction)(m.bind);
    }
    catch (error) {
        return false;
    }
}
exports.isRight = isRight;
function expectEither(x) {
    if (!isLeft(x) && !isRight(x)) {
        throw new Error('Expected Either instance but got ' + String(x));
    }
    return x;
}
