"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNothing = exports.isJust = exports.fromMaybe = exports.maybe = exports.Just = exports.Nothing = exports.NOTHING_TAG = exports.JUST_TAG = void 0;
const assertions_1 = require("../assertions/assertions");
const utils_1 = require("../utils/utils");
exports.JUST_TAG = Symbol('TAG: Maybe.Just');
exports.NOTHING_TAG = Symbol('TAG: Maybe.Nothing');
/**
 * Data instance for the Nothing {@link Maybe} member.
 */
exports.Nothing = Object.freeze({
    TAG: exports.NOTHING_TAG,
    fmap: () => exports.Nothing,
    apply: () => exports.Nothing,
    bind: () => exports.Nothing,
    toString: () => `Nothing`,
});
/**
 * Data constructor for the Just {@link Maybe} member.
 */
const Just = (x) => Object.freeze({
    TAG: exports.JUST_TAG,
    value: x,
    fmap: (f) => (0, exports.Just)(f(x)),
    apply: (mf) => expectMaybe(mf).fmap((f) => f(x)),
    bind: (f) => expectMaybe(f(x)),
    toString: () => `Just (${(0, utils_1.show)(x)})`,
});
exports.Just = Just;
/**
 * Takes a default value, a function, and a Maybe value. If the Maybe value is Nothing, the function returns the
 * default value. Otherwise, it applies the function to the value inside the Just and returns the result.
 */
const maybe = (x) => (f) => (m) => isJust(m) ? f(m.value) : x;
exports.maybe = maybe;
/**
 * Takes a default value and returns a new function that takes a Maybe. If the Maybe is {@link Nothing}
 * it returns the default value. If the Maybe is {@link Just} it returns the contained value.
 */
const fromMaybe = (x) => (m) => isJust(m) ? m.value : x;
exports.fromMaybe = fromMaybe;
/**
 * Tells if the given value is a {@link Just} instance of Maybe.
 */
function isJust(m) {
    try {
        return m.TAG === exports.JUST_TAG
            && m.hasOwnProperty('value')
            && (0, assertions_1.isFunction)(m.fmap)
            && (0, assertions_1.isFunction)(m.apply)
            && (0, assertions_1.isFunction)(m.bind);
    }
    catch (error) {
        return false;
    }
}
exports.isJust = isJust;
/**
 * Tells if the given value is a {@link Nothing} instance of Maybe.
 */
function isNothing(m) {
    try {
        return m.TAG === exports.NOTHING_TAG
            && (0, assertions_1.isFunction)(m.fmap)
            && (0, assertions_1.isFunction)(m.apply)
            && (0, assertions_1.isFunction)(m.bind);
    }
    catch (error) {
        return false;
    }
}
exports.isNothing = isNothing;
function expectMaybe(x) {
    if (!isJust(x) && !isNothing(x)) {
        throw new Error('Expected maybe value but got ' + String(x));
    }
    return x;
}
