import { isFunction } from '../assertions/assertions';
import { show } from '../utils/utils';
export const LEFT_TAG = Symbol('TAG: Either.Left');
export const RIGHT_TAG = Symbol('TAG: Either.Right');
export const Left = (x) => ({
    TAG: LEFT_TAG,
    value: x,
    fmap: () => Left(x),
    apply: () => Left(x),
    bind: () => Left(x),
    // @ts-ignore
    toString: () => `Left (${show(x)})`,
});
export const Right = (x) => ({
    TAG: RIGHT_TAG,
    value: x,
    fmap: (f) => Right(f(x)),
    apply: (ef) => expectEither(ef).fmap((f) => f(x)),
    bind: (f) => expectEither(f(x)),
    // @ts-ignore
    toString: () => `Right (${show(x)})`,
});
/**
 * Tells if the given value is a {@link Just} instance of Maybe.
 */
export function isLeft(m) {
    try {
        return m.TAG === LEFT_TAG
            && m.hasOwnProperty('value')
            && isFunction(m.fmap)
            && isFunction(m.apply)
            && isFunction(m.bind);
    }
    catch (error) {
        return false;
    }
}
/**
 * Tells if the given value is a {@link Nothing} instance of Maybe.
 */
export function isRight(m) {
    try {
        return m.TAG === RIGHT_TAG
            && isFunction(m.fmap)
            && isFunction(m.apply)
            && isFunction(m.bind);
    }
    catch (error) {
        return false;
    }
}
function expectEither(x) {
    if (!isLeft(x) && !isRight(x)) {
        throw new Error('Expected Either instance but got ' + String(x));
    }
    return x;
}
