import type { Fn } from '../types';
export declare const JUST_TAG: unique symbol;
export declare const NOTHING_TAG: unique symbol;
export type Maybe<T> = ({
    readonly TAG: typeof JUST_TAG;
    readonly value: T;
} | {
    readonly TAG: typeof NOTHING_TAG;
}) & {
    fmap<U>(f: Fn<T, U>): Maybe<U>;
    apply<U>(f: Maybe<Fn<T, U>>): Maybe<U>;
    bind<U>(f: Fn<T, Maybe<U>>): Maybe<U>;
};
/**
 * Data instance for the Nothing {@link Maybe} member.
 */
export declare const Nothing: Maybe<never>;
/**
 * Data constructor for the Just {@link Maybe} member.
 */
export declare const Just: <T>(x: T) => Maybe<T>;
/**
 * Takes a default value, a function, and a Maybe value. If the Maybe value is Nothing, the function returns the
 * default value. Otherwise, it applies the function to the value inside the Just and returns the result.
 */
export declare const maybe: <U>(x: U) => <T>(f: Fn<T, U>) => (m: Maybe<T>) => U;
/**
 * Takes a default value and returns a new function that takes a Maybe. If the Maybe is {@link Nothing}
 * it returns the default value. If the Maybe is {@link Just} it returns the contained value.
 */
export declare const fromMaybe: <T>(x: T) => (m: Maybe<T>) => T;
/**
 * Tells if the given value is a {@link Just} instance of Maybe.
 */
export declare function isJust<T>(m: Maybe<T>): m is Maybe<T> & {
    TAG: typeof JUST_TAG;
    value: T;
};
/**
 * Tells if the given value is a {@link Nothing} instance of Maybe.
 */
export declare function isNothing<T>(m: Maybe<T>): m is Maybe<T> & {
    TAG: typeof NOTHING_TAG;
};
